\ Interrupt drivern serial I/O for single chip 80C196 systems

((
Copyright (C) 1990..1991

MicroProcessor Engineering
133/133a Hill Lane
Shirley
Southampton SO1 5AF

tel: (+44) 703 631441
fax: (+44) 703 339691

The serial driver for an Umbilical Forth system must put the processor
to sleep while waiting for a character from the serial line. This is needed
to allow an EPROM emulator to write data into the EPROM space. By using
an interrupt driven serial line, and putting the processor to sleep while
waiting, the CPU does not access EPROM memory while the host PC is writing
into the EPROM emulator.

If the processor cannot be put to sleep, or a bit-banging UART is being
used, the polling loop must be executed from RAM to avoid accessing
EPROM space.

May 1991	Converted to text files
		Drivers moved to DRIVERS directory
))

\ 80C196 internal serial port                      sfp 05/02/89

hex
\ bits in the SP_STAT register
040 equ ri-bit          \ receiver ready bit
020 equ ti-bit          \ transmitter ready bit
0BF equ /ri-bit         \ reset mask
0DF equ /ti-bit         \ reset mask
6   equ ri-bit#
5   equ ti-bit#


\ baud rate equates                                sfp 18/12/90

hex

\ 12 MHz baud rates for 196 - divide lsb by 4 for 96
\ 0804D equ baud9600
\ 0809B equ baud4800
\ 08138 equ baud2400
\ 08270 equ baud1200

\ 10 MHz baud rates for 196KB - divide lsb by 4 for 96
  08040 equ baud9600
  08081 equ baud4800
  08103 equ baud2400
  08208 equ baud1200

\ 8 MHz baud rates for 196 - divide lsb by 4 for 96
\ 08033 equ baud9600
\ 08067 equ baud4800
\ 080CF equ baud2400
\ 081A0 equ baud1200

\ init-ser

hex

code init-ser   \ -- ; initialise serial port
  jbc ios1 5 $				\ wait for UART
  jbc ios1 5 $				\ by waiting for Timer 1 Overflow
  jbc ios1 5 $
  ldb baudrate # sysbaud .lo		\ initialise baud rate
  ldb baudrate # sysbaud .hi		\   as two bytes to same address
  ldb ioc1 # 020			\ IOC1.5 to 1 to select TXD
  ldb sp_con # 09			\ REN=1, mode=1
  ldb spshadow sp_stat			\ read status to reset RI and set TI
  orb spshadow # ti-bit                 \ mark ready to xmit
  orb int_mask_1 # 003			\ enable RI/TI ints; INT_MASK_1.0/1
  ei					\ and enable interrupts
  next,   end-code

\ (emit) (key) (key?)

assembler
l: ti-isr
  pushf					\ disable interrupts
  orb spshadow sp_stat			\ OR status register into shadow
  popf					\ restore interrupts
  ret
forth
ti-isr 02030 !				\ set interrupt vector

assembler
l: ri-isr	\ entry point for receive ISR
  pushf					\ disable interrupts
  orb spshadow sp_stat			\ read status register
  ldb rxchar sbuf			\ read character
  popf					\ restore interrupts
  ret
forth
ri-isr 02032 !				\ set up interrupt vector
  
code (emit)     \ char -- ; transmit character
l: pemit1				\ interrupt will set bit
    di					\ use of shadow regs. must be locked
    jbs spshadow ti-bit# pemit2		\   test receive status
    ei  nop
    idlpd # 1				\   power down
    sjmp pemit1				\   test again
l: pemit2
  andb spshadow # /ti-bit               \ reset ti bit in shadow
  ldb sbuf tos.l                        \ output character
  ei
  pop tos				\ restore TOS
  next,
end-code

code (key)      \ -- char ; receive a character
  push tos				\ save previous TOS
l: pkey1				\ interrupt will set bit
    di					\ use of shadow regs. must be locked
    jbs spshadow ri-bit# pkey2          \   test receive status
    ei  nop
    idlpd # 1				\   power down
    sjmp pkey1				\   test again
l: pkey2
  andb spshadow # /ri-bit               \ reset ri bit in shadow
  ldbze tos rxchar			\ receive character
  ei
  next,   end-code

code (key?)     \ -- t/f ; true if character available
  push tos                              \ preserve old TOS
  ld tos # -1                           \ assume success
  jbs spshadow ri-bit# pkeyq1           \ skip if char ready
  clr tos                               \ set accordingly
l: pkeyq1
  next,   end-code                      \ all done

\ Umbilical Forth i/o

hex

: key?          \ -- t/f
  key?-cmnd>h send-message  ;

: key           \ -- char
  key-cmnd>h send-message  ;

: emit          \ char -- ; send character
  emit-cmnd>h send-message  1 out c+!  ;

\ The code below avoids using DO ... LOOP and their 3 items on the R. stack
: type          \ addr len -- ; display string
  begin  ?dup				\ while characters left
    while  over c@ emit			\ display them
	   swap 1+ swap 1-		\ adjust address and length
  repeat
  drop  ;

: cr            \ --
  0A emit   0D emit  0 out c!  ;

\ all done

decimal

;p
