((

	TARGPAGES.FTH

	J Lee :- 26/6/90

	MicroProcessor Engineering Ltd
	133/133a Hill Lane
	Southampton
	SO1 5AF

	tele:	(0703) 631441
	inter:	+44 703 631441


	This file contains code for both symbol pages and target pages.

))



only forth also c-c definitions
decimal



\ Build-Page .. J Lee -- 14/08/90
\


\ pcb page.pcb	page.pcb pathname apage.img

global-var Page#		\ page number for next page
	1 page# !(g)		\ start at 1 (page 0 is kernel)


: Finish-File	( -- ; close down old file )
  flg.upd @		\ block updated?
  If			\ y: finish old image file.
    i.write flg.upd off
  Endif
  i.close
  blk.cur @  blk.cur(p) !(p)	\ preserve current block.
  flg.upd @ flg.upd(p) !(p)  ;	\ and update-flag



: Preserve-Page	( -- ; preserve page variables )
  code-page-open @(g)  in-umbilical @(g) 0=  AND
  If
    curr-code-page @(g) in-emulator? 0=
    If
      finish-file
    Endif
  Endif
  curr-page@ old-curr-page !(g)
  voc-seg @(g) voc-seg(p) !(p)  ;


: Restore-Page	( -- ; restore page variable )
	old-curr-page @(g) curr-page!
	voc-seg(p) @(p) voc-seg !(g)
	blk.cur(p) @(p) blk.cur !
	flg.upd(p) @(p) flg.upd !  ;


decimal

: Set-Page-Name	( $addr page# -- ; add page name to page table )
	>page-table page.name over c@ 1+
	32 min	\ max length of $
	cmove  ;	\ copy to page-table


: Find-Page$	( $addr -- $addr/Page# t/f )
  FALSE			\ assume failure
  #Pages 0			\ for all pages
  ?Do
    i >page-table page.chunk @(g)		\ page exists?
    i data-page?  OR
    If
      over dup  i >page-table page.name  swap c@ 1+  s= \ right page?
      If
        nip					\ discard $addr
	NOT					\ set FALSE to TRUE
	i swap				\ leave page#
	LEAVE				\ and leave loop
      Endif
    Endif
  Loop  ;


: Set-File	( -- )
	img.pcb i.open abort" Can't open PAGE file"  \ and open
	i.read  ;	\ read in block


: Set-Page-File	( -- ; set filename for page )
	Page# @(g) >page-table page.name	\ get page name
	img.pcb set-pathname			\ set pathname in pcb
	"" .IMG" img.pcb set-ext  ;		\ and add extension


: Start-Page	( symbol-page# -- )
    dup curr-page!				\ set current page
    voc-seg(p) @(p) voc-seg !(g)
    in-umbilical @(g) 0=  over in-emulator? 0=  AND  \ not umb. or emulator
    If					\ open image file
        >page-table page.name img.pcb set-pathname
        "" .IMG" img.pcb set-ext
        set-file			\ open file ready
        code-page-open on(g)
    Else
        drop				\ page#
    Endif  ;


: Build-Page	( page-id $addr -- ; build page with voc name at $addr )
  Page# @(g) dup init-page	\ build symbol page
  2dup set-page-name		\ add name to page-table
  >page-table page.id		\ set page.id in page-table
  rot swap !(g)	
  cs> swap $make-voc		\ build voc in symbols
  curr-page-chunk last'(s)@	\ link voc in symbols
  2dup Page-Voc(s) 2!(p)	\ set voc. as 'root' symbol voc.
  2dup voc-chunk+ !(p)
  2dup context(s) 2!(p)
  set-definitions(s)  
  set-page-file
  1 Page# +!(g)  ;			\ set page# for next page





\ Page-Vocabulary .. J Lee -- 17/12/90
\


hex


020 constant /page-header

004 offset page-voc




c-c only forth also c-c definitions


: Paged-Ram-Voc(t)	( #threads t-cfa -- ^threads )
  here(t) 4 + ,(t)
  Voc-Link(t) @(g) ,(t)         \ update voc-link(t) with t-cfa
\  Voc-Link(t) !(g)              \
  drop						\ don't alter VOC-LINK(T)
  here(t) >r                    \ save ^threads
  dup ,(t) 0 ?Do  0 ,(t)  Loop  \ zero alloted threads
  r> cell+  ;                   \


: Paged-Voc(T)	( )
  "" VC" here(t) .log                 \ log it to user
  here(t)                             \
  target VOCABULARY Compile-run       \ vocabulary run-time-ref
  cur-#threads(t) swap                \ #threads for this voc.
  paged-Ram-voc(t)  ;


: Paged-Voc(H)	( -- ; build new vocab )
  CREATE  (make-voc)			\ build new host vocab
  latest-chunk @(v) last'(s)@	\ return seg offset of symbol
  create-voc-chunk			\ create a new voc chunk
  -rot link-voc-chunk			\ set pointer in voc symbol
  paged-VOC(T)  				\ build target vocab
  'latest(s) 2@(v) t-vp+ !(s)l	\ stash CONTEXT addr
  'latest(s) 2@(v)
  status+ mark-voc  ;			\ mark symbol as vocab



only forth also c-c also cc/i


: Paged-Vocabulary	CC/I	( -- <voc-name> )
  ?0-even  cs> get-token find(t)
  If
    2dup voc?
    If
      create                  \ resolve symbol
      paged-voc(t)            \ make target
      -rot t-vp+ !(s)l        \ stash address
    Else
      drop paged-voc(h)
    Endif
  Else
    paged-voc(h)
  Endif  ;



c-c  only forth also c-c definitions




\ Code-Page .. J Lee -- 18/10/90
\

only forth also c-c also cc/i

: Code-Page  cc/i	( start-addr end-addr page-id -- ; <pagename> )
  preserve-page
  get-token build-page 
  curr-page@ >page-table page.end !(g)		\ set ROM-end
  dup curr-page@ >page-table page.start !(g)	\ set page start in page table
  dup dp(t) !(p)  o-i !(p)            		\ DP and O-I = page addr.
  in-umbilical @(g) 0=
  If
    curr-page@ >page-table
    dup page.start @(g)  swap page.end @(g)
    img.pcb write-blocks  			\ create new image file
  Endif
  /page-header allot(t)
  restore-page  ;


: Data-Page  cc/i	( start-addr end-addr page-id -- ; <pagename> )
  get-token Page# @(g) dup >r
  set-page-name                      \ add name to page table
  r@ >page-table page.id !(g)        \ set page.id in page table
  r@ >page-table page.end !(g)	     \ set page-end
  dup r@ >page-table page.start !(g) \ set page start in page table
  r@ >page-table page.rp !(g)        \ set RAM pointer
  r> mark-data-page  		     \ mark as a data page
  1 Page# +!(g)  ;                   \ step on page#


: Kernel	cc/i	( start-addr end-addr -- <pagename> )
  preserve-page
  get-token dup 0 set-page-name	\ add name to page table
  img.pcb set-pathname  "" .IMG" img.pcb set-ext	\ set image file name
  0 0 >page-table page.id !(g)		\ set 0 as page-id
  2dup img.pcb write-blocks   \   so create file
  0 >page-table page.end !(g)		\ mark ROM end
  dup 0 >page-table page.start !(g)	\ set page start in page table
  dup Dp(t) !(p)  dup O-I !(p)
  the-label ORIGIN 2dup status+ mark-resolved !(s)l
  restore-page  ;


: Kernel-Page cc/i	( start-addr end-addr page-id -- <pagename> )
  -rot [+interpreter] kernel [previous]
  0 >page-table page.id !(g)  ;		\ set page-id


: Kernel-Ram cc/i    ( start-addr end-addr page-id -- <pagename> )
  rot dup  the-label RAM-START 2dup status+ mark-resolved !(s)l  -rot
  >r  2dup swap - ?dup  If  initialised-ram  Endif  r>
  rom/ram                       \ ROM / RAM based target
  page# @(g) kernel-ram-page !(g)       \ note intialised RAM page
  [+interpreter] data-page [previous]  ;


: Use-Code	cc/i	( -- ; <pagename> )
  Code-Page-Open @(g)  in-umbilical @(g) 0=  AND
  If
    curr-code-page @(g) in-emulator? 0=
    If
      finish-file
    Endif
  Endif
  get-token find-page$
  If
    dup curr-code-page !(g)			\ preserve this symbol page
    dup start-page
  Else
    ." Can't find page " $. abort		\ Error message
  Endif
  in-emulator?                          \ output to emulator?
  If
    >emulator                           \ y:  switch in emulator drivers
  Else
    >disk                               \ n:  switch in disk drivers
  Endif  ;



: Use-Data	cc/i	( -- ; <pagename> )
  get-token find-page$
  If
    curr-data-page !(g)				\ preserve this symbol page
  Else
    ." Can't find page " $. abort		\ Error message
  Endif  ;


: Finis-Code-Page  cc/i	( -- )
  Page-Voc(s) 2@(p) 2dup resolved?	\ vocabulary declared?
  If
    @(s)l					\ retrieve addr
  Else
    2drop 0					\ null
  Endif
  o-i @(p) page-voc !(t)        \ set in header
  (finis-voc)				\ tidy up page vocabulary
  ?resolve					\ check words for this page
  ?rom-end  ;



\ Emulator support for pages .. J Lee -- 17/4/91
\


c-c only forth also c-c definitions

: Del-Emu-File  ( page# -- ; delete image file since using emulator ) 
  >page-table page.name img.pcb set-pathname  \ get file name
  "" .IMG" img.pcb set-ext                    \ and add extension
  img.pcb p-name delete-path  \ delete file
  If                                          \ error?
    cr ." can't delete: " img.pcb .pcb ABORT  \ y: abort!
  Endif  ; 


: Set-Emu-Offset  ( d-offset page# -- ; set emulator offset in page-table )
  >page-table page.epem 2!(g)  ;


only forth also c-c also cc/i
  
: In-Emulator  cc/i  ( offset -- <page> ; mark <page> as in emulator at offset )
  emulator? 0=  abort" can't find emulator"
  0                             \ convert offset to a +ve double
  get-token find-page$          \ find the page
  If                            \ found?
    dup mark-emulator           \ y: mark page as in emulator
    dup del-emu-file            \    delete blank image file
    set-emu-offset              \    set offset into emulator
  Else                                  
    cr ." Can't find PAGE " $. ABORT    \ n: error!
  Endif  ;


c-c only forth also c-c definitions
