\ 80C196KC PowerForth code definitions
\ released for use without royalty with the MPE Forth Cross Compiler

pto

Copyright (c) 1987..1992

MicroProcessor Engineering
133 Hill Lane
Southampton SO1 5AF
England

tel: +44 703 631441
fax: +44 703 339691
net: mpe@compulink.co.uk

Note that the 80C196KC is required as this file uses the DIV DIVU DJNZW
and XCH instructions.

\ Cold and Warm start

hex

ASSEMBLER

up equ up                       \ make UP generally available

\ perform 80C196KC initialisation in the approved style
L: ECLD
  di                            \ disable interrupts
  dpts				\ disable PTS
  clr ax   begin,  nop  nop  dec ax  eq, until, \ wait a while

\
\ the following line is needed by the MPE 80C196 PowerBoard
  ld ax em 2-   st ax em 2-     \ wake up DS1210 by accessing RAM

  ld sp # init-s0               \ set data stack
  stb r0 int_mask_1		\   secondary interrupt mask
  push r0   popf		\ clear flags and IMASK
  stb r0 ioc1			\ and IOC1

  ld ip # cld1                  \ set IP
l: ecld/wrm
\
\ the following line is needed by the MPE 80C196 PowerBoard
  andb port1 # 0FE		\ disable memory mapper

  ld sp # init-s0               \ set data stack
  ld rp # init-r0               \ set return stack
  ld up # init-u0               \ set user pointer
  ldb task# # 080		\ turn multitasker OFF
  next,				\ go do it
L: EWRM
  ld ip # WRM1
  sjmp ecld/wrm 
forth


\ interpreters .................................   sfp 12/01/89

ASSEMBLER
l: dodoes
  sub rp # 2   st ip rp []              \ put IP on R. stack
  pop ip                                \ get new IP
l: docreate
  pop ax                                \ get new PFA
  push tos                              \ save old TOS
  ld tos ax                             \ form new TOS
l: next
  ld ax ip []+                          \ fetch CFA, incr. IP
  br ax []                              \ jmp to it
FORTH

\ execute lit branch ?branch                       sfp 12/01/89

\ internal
CODE LIT        push tos   ld tos ip []+
                next,   end-code

CODE BRANCH     \ -- ; target address is in-line word
L: BRAN1        ld ip ip []   next,   end-code

CODE ?BRANCH    cmp tos r0   pop tos   je bran1
L: 0BRAN1       add ip # 2   next,     END-CODE

CODE EXECUTE    ld ax tos   pop tos   br ax []   END-CODE


\ (of)                                             sfp 08/01/89

CODE (OF)       \ n1 n2 -- ; n1 n2 -- n1 [branch taken]
  cmp tos sp []   pop tos   jne bran1   \ compare n1 n2, drop n2
  pop tos                               \ equal, drop n1 too
  add ip # 2   next,                    \ skip branch offset
end-code


\ (loop) (+loop)                                   sfp 10/01/89

CODE (LOOP)     \ -- ; affects return stack when loop ends
  ld ax rp []   inc ax   jv loop-exit   \ test index-limit+8000
L: LOOP-CONT
  st ax rp []   ld ip ip []             \ round again
  next,   end-code

CODE (+LOOP)    \ n -- ; see (LOOP)
  add ax tos & rp []  pop tos           \ add increment
  jnv loop-cont                         \ check, again if NV
L: LOOP-EXIT
  add rp # 6                            \ update RP
  add ip # 2   next,   end-code         \ skip branch offset


\ (do) (?do)                                       sfp 08/01/89

hex

CODE (DO)       \ limit start --
l: pdo1
  sub rp # 6                            \ 3 new items on stac
  ld ax ip []+   st ax 4 rp ,[]         \ stash end address
  pop ax   add ax # 08000   st ax 2 rp ,[] \ save limit+08000h
  sub tos ax   st tos rp []             \ start-limit-08000h
  pop tos                               \ restore stack
  next,  end-code

CODE (?DO)      \ limit start --
  cmp tos sp []   jne pdo1              \ do loop if different
  add sp # 2   pop tos                  \ 2drop, maintain TOS
  ld ip ip []   next,   end-code        \ take branch

\ (leave) (?leave) ..........................      sfp 02/01/89

CODE (LEAVE)    \ -- ; leave DO ... LOOP
l: leave1
  ld ip 4 rp ,[]                        \ get exit address
  add rp # 6                            \ update r. stack
  next,   end-code                      \

CODE (?LEAVE)   \ n -- ; leave DO ... LOOP if n<>0
  cmp tos r0   pop tos   jne leave1    \ test TOS
  next,   end-code

\ external


\ i j                                              sfp 21/12/88

code i                  \ -- n ; return DO ... LOOP index
  push tos
  ld tos rp []                          \ start-limit-08000+inc
  add tos 2 rp ,[]                      \ limit+08000
  next,   end-code                      \

code j                  \ -- n ; return index of next outer loop
  push tos                              \ save old TOS
  ld tos 6 rp ,[]   add tos 8 rp ,[]    \ add data
  next,   end-code                      \


\ digit                                            sfp 11/01/89

hex

code digit      \ char base -- n true / false
  ld cx sp []                           \ cx=char, tos=base
  subb cl # 030   jlt digit2            \ false if char<'0'
  cmpb cl # 0A   jlt digit0             \ 0..9 if < 0A
  cmpb cl # 011   jlt digit2            \ false if 0A..010
  subb cl # 7                           \ 011 -> 0A
l: digit0
  cmpb cl tos.l   jge digit2            \ false if n>=base
  st cx sp []   ld tos # -1             \ store n, form true flg
  next,
l: digit2
  add sp # 2   clr tos                  \ drop, form false flag
  next,   end-code


\ /string noop

code /string                    \ addr len n -- addr+n len-n
  ld ax tos   pop tos
  sub tos ax                            \ update length
  add ax sp []   st ax sp []            \ update address
  next,   end-code

code noop                               \ -- ; dummy
  next,   end-code

\ min max                                          sfp 21/12/88

code min        \ n1 n2 -- n1|n2 ; return lesser of n1 and n2
  cmp tos sp []   jge minx              \ tos-nos
  add sp # 2   next,                    \ use TOS
l: minx
  pop tos   next,   end-code            \ use NOS

code max        \ n2 n1 -- n1|n2 ; return greater of n1 and n2
  cmp tos sp []   jle maxx              \ tos-nos
  add sp # 2   next,                    \ use TOS
l: maxx
  pop tos   next,   end-code            \ use NOS


\ cmove cmove>

CODE CMOVE      \ source dest len -- ; copy memory areas
  pop cx   pop bx                       \ get parameters
  cmp tos r0                            \ skip if 00
  ne, if,
l: cmove1
          ldb al bx []+   stb al cx []+
          djnzw tos cmove1
      endif,
  pop tos   next,   end-code

CODE CMOVE>     \ source dest len -- ; copy memory areas
  add cx tos & sp []+                   \ form dest address
  add bx tos & sp []+                   \ form source address
  cmp tos r0                            \ skip if 00
  ne, if,
l: cmove>1
           dec bx   dec cx
           ldb al bx []   stb al cx []
           djnzw tos cmove>1
      endif,
  pop tos   next,   end-code

\ fill                                             sfp 08/11/88

: FILL          \ addr len char
  SWAP >R                               \ save length
  OVER C!                               \ set first character
  DUP 1+  R> 1- CMOVE  ;                \ copy n-1 chars

code s=         \ addr1 addr2 count -- flag
  pop bx   pop cx                       \ get parameters
  cmp tos r0   je se2			\ check for zero count
l: se1
  ldb al bx []+   cmpb al cx []+	\ compare a byte
  jne se3				\ exit if different
  djnzw tos se1				\ test counter
l: se2
  ld tos # -1   next,			\ return true
l: se3
  clr tos				\ return false
  next,   end-code

\ um*                                              sfp 21/12/88

code um*        \ u1 u2 -- ud ; unsigned multiply
  mulu tos sp []
  st tos sp []   ld tos tosext
  next,   end-code

code *          \ n1 n2 -- n3 ; signed multiply
  mul tos sp []+   next,   end-code

code m*         \ n1 n2 -- d
  mul tos sp []
  st tos sp []   ld tos tosext
  next,   end-code

\ um/mod sm/mod

hex

code um/mod     \ u32 u16 -- urem uquot
  ld bx tos                             \ save divisor
  pop tosext   pop tos                  \ get dividend
  divu tos bx				\ perform division
  jv um/mod-oflow			\ overflow check
  push tosext				\ save remainder, quotient in TOS
  next,
l: um/mod-oflow
  ld tos # 0FFFF   push tos
  next,  end-code

code sm/mod     \ n32 n16 -- rem quot
  ld bx tos                             \ save divisor
  pop tosext   pop tos                  \ get dividend
  div tos bx				\ perform division
  jv sm/mod-oflow
  push tosext
  next,
l: sm/mod-oflow
  ld tos # 07FFF   push tos
  next,  end-code

\ m/mod /mod

code m/mod	\ n32 n16 -- rem quot
  ld bx tos                             \ save divisor
  pop tosext   pop tos                  \ get dividend
  ld ax bx   xor ax tosext		\ form sign of quotient
  div tos bx				\ perform division
  jv m/mod-oflow			\ overflow check
  or ax ax				\ check sign of quotient
  lt, if,  or tosext tosext		\ if remainder exists
	   ne, if,  dec tos		\ decrement quotient
                    add tosext bx	\ add divisor to remainder
	       endif,
      endif,
  push tosext				\ save remainder
  next,
l: m/mod-oflow
  ld tos # 07FFF   push tos
  next,  end-code

: /mod		\ n1 n2 -- rem quot
  >r s>d r> m/mod   ;

\ / u/ mod */mod */ mu/mod

code /		\ n1 n2 -- quot
  ld bx tos                             \ save divisor
  pop tos   ext tos			\ get dividend
  ld ax bx   xor ax tosext		\ form sign of quotient
  div tos bx				\ perform division
  jv /-oflow				\ overflow check
  or ax ax				\ check sign of quotient
  lt, if,  or tosext tosext		\ if remainder exists
	   ne, if,  dec tos  endif,	\ decrement quotient
      endif,
  next,
l: /-oflow
  ld tos # 07FFF
  next,  end-code

code u/		\ u1 u2 -- uquot
  ld bx tos                             \ save divisor
  pop tos   ext tos			\ get dividend
  divu tos bx				\ perform division
  jv u/-oflow				\ overflow check
  next,
l: u/-oflow
  ld tos # 0FFFF
  next,  end-code

code mod	\ n1 n2 -- rem
  ld bx tos                             \ save divisor
  pop tos   ext tos			\ get dividend
  ld ax bx   xor ax tosext		\ form sign of quotient
  div tos bx				\ perform division
  jv mod-oflow				\ overflow check
  or ax ax				\ check sign of quotient
  lt, if,  or tosext tosext		\ if remainder exists
	   ne, if,  add tosext bx	\ add divisor to remainder
	       endif,
      endif,
  ld tos tosext				\ save remainder
  next,
l: mod-oflow
  ld tos # 07FFF
  next,  end-code

: */mod		\ n1 n2 n3 -- rem quot ; n1*n2/n3 with 32-bit intermediate
  >r m* r> m/mod  ;

: */		\ n1 n2 n3 -- quot ; n1*n2/n3 with 32-bit intermediate
  */mod nip  ;

: mu/mod	\ ud n -- urem udquot ; used in number conversion
  >r 0 r@ um/mod r> swap >r um/mod r> ;

\ and or xor not

CODE AND        \ n1 n2 -- n3
  and tos sp []+   next,   end-code

CODE OR         \ n1 n2 -- n3
  or tos sp []+   next,   end-code

CODE XOR        \ n1 n2 -- n3
  xor tos sp []+   next,   end-code

CODE NOT        \ n1 -- n2
  not tos   next,   end-code


\ sp@ sp! rp@ rp!

CODE SP@        \ -- n ; return data stack pointer
  push tos   ld tos sp   next,   END-CODE

CODE SP!        \ n -- ; set data stack pointer
  ld sp tos   pop tos   next,   end-code

CODE RP@        \ -- n ; get return stack pointer
  push tos   ld tos rp   next,   end-code

CODE RP!        \ n -- ; set return stack pointer
  ld rp tos   pop tos   next,   end-code


\ exit >r r> r@

code exit	\ -- ; return from colon definition
  ld ip rp []+   next,   end-code

CODE >R         \ n -- ; copy to return stack
  sub rp # 2   st tos rp []   pop tos
  next,   end-code

CODE R>         \ -- n ; from return stack
  push tos   ld tos rp []+
  next,   end-code

CODE R@         \ -- n ; copy of return stack
  push tos   ld tos rp []   next,   end-code
\ 0< 0> 0<> 0=

CODE 0<		\ n -- t/f
  cmp tos 0   jlt set-tru   sjmp set-fls   end-code

CODE 0>		\ n -- t/f
  cmp tos 0   jgt set-tru   sjmp set-fls   end-code

CODE 0<>	\ n -- t/f
  cmp tos 0   jne set-tru   next,   end-code

CODE 0=		\ n -- t/f
  cmp tos r0   jne set-fls
L: SET-TRU
  ld tos # -1   next,
L: SET-FLS
  clr tos   next,   END-CODE


\ = <> u< u> < >

CODE =		\ n1 n2 -- t/f
  cmp tos sp []+   je set-tru   sjmp set-fls   end-code

CODE <>		\ n1 n2 -- t/f
  cmp tos sp []+   jne set-tru   sjmp set-fls   end-code

CODE U<		\ n1 n2 -- t/f
  cmp tos sp []+   jh set-tru   sjmp set-fls   end-code

CODE U>		\ n1 n2 -- t/f
  cmp tos sp []+   jh set-fls   je set-fls   sjmp set-tru
  end-code

CODE <          \ n1 n2 -- t/f
  cmp tos sp []+   jgt set-tru   sjmp set-fls  end-code

CODE >          \ n1 n2 -- t/f
  cmp tos sp []+   jlt set-tru   sjmp set-fls   end-code

CODE <=		\ n1 n2 -- t/f
  cmp tos sp []+   jge set-tru   sjmp set-fls  end-code

CODE >=		\ n1 n2 -- t/f
  cmp tos sp []+   jle set-tru   sjmp set-fls   end-code

\ - +                                              sfp 02/01/89

CODE -          \ n1 n2 -- n1-n2
  pop ax   sub tos ax & tos
  next,   end-code

CODE +          \ n1 n2 n1+n2
  add tos sp []+   next,   end-code


\ d+ d-

CODE D+         \ d1 d2 -- d3
  ld tosext sp []+                      \ get d2.l
  add tosext 2 sp ,[]                   \ d3.l=d2.l+d1.l
  addc tos sp []+                       \ d3.h=d2.h+d1.h
  st tosext sp []                       \ stash d3.l
  next,   END-CODE

CODE D-         \ d1 d2 -- d1-d2
  ld tosext sp []+                      \ d2.l
  ld ax sp []+   ld bx sp []            \ ax=d1.h bx=d1.l
  sub bx tosext   subc ax tos           \ perform subtraction
  st ax tos   st bx sp []               \ stash result
  next,   end-code                      \


\ negate ?negate abs

CODE negate     \ n1 -- -n1
l: neg1
  neg tos   next,   end-code

code ?negate    \ n1 t/f -- n1/-n1
  cmp tos r0   pop tos   jlt neg1
  next,   end-code

code abs        \ n1 --  |n1|
  cmp tos r0   jlt neg1
  next,   end-code


\ dnegate ?dnegate dabs

CODE dnegate    \ d1 -- -d1
l: dneg1
  ld tosext sp []   not tos   neg tosext
  e, if,  inc tos  endif,
  st tosext sp []   next,   end-code

code ?dnegate   \ d1 t/f -- d1/-d1
  cmp tos r0   pop tos   jlt dneg1
  next,   end-code

code dabs       \ d1 -- |d1|
  cmp tos r0   jlt dneg1
  next,   end-code

\ over 2over

CODE OVER       \ n1 n2 -- n1 n2 n1
  push tos   ld tos 2 sp ,[]
  next,   end-code

CODE 2OVER      \ n1 n2 n3 n4 -- n1 n2 n3 n4 n1 n2
  push tos   ld tos 6 sp ,[]
  push tos   ld tos 6 sp ,[]
  next,   end-code


\ drop 2drop

CODE DROP       \ n1 --
  pop tos   next,   end-code

CODE 2DROP      \ n1 n2 --
  add sp # 2   pop tos   next,   end-code


\ swap 2swap

CODE SWAP       \ n1 n2 -- n2 n1
  xch tos 0 sp ,[]   next,   end-code

CODE 2SWAP      \ n1 n2 n3 n4 -- n3 n4 n1 n2
  pop tosext				\ tos = n4, tosext = n3
  xch tos 0 sp ,[]			\ exchange n2 and n4
  xch tosext 2 sp ,[]			\ exchange n1 and n3
  push tosext				\ put n1 back
  next,   end-code


\ dup 2dup ?dup

CODE DUP        \ n1 -- n1 n1
l: dup1
  push tos   next,   end-code

CODE 2DUP       \ n1 n2 -- n1 n2 n1 n2
  ld tosext sp []
  push tos   push tosext
  next,   end-code

code ?dup       \ n1 -- n1 [n1]
  cmp tos r0   jne dup1
  next,   end-code

\ rot -rot s->d

CODE ROT        \ n1 n2 n3 -- n2 n3 n1
  pop ax   pop bx
  push ax   push tos   ld tos bx
  next,   end-code

code -rot       \ n1 n2 n3 -- n3 n1 n2
  pop ax   pop bx
  push tos   push bx   ld tos ax
  next,   end-code

CODE S>D        \ n -- d
  push tos   ext tos   ld tos tosext
  next,   end-code


\ nip tuck pick roll

code nip        \ n1 n2 -- n2
  add sp # 2   next,   end-code

code tuck       \ n1 n2 -- n2 n1 n2
  pop ax   push tos   push ax
  next,   end-code

code pick       \ item -- n
  shl tos # 1   add bx sp & tos
  ld tos bx []
  next,   end-code

: ROLL          \ n1 n2 .. nk n -- n2 .. nk n1
		\ DO NOT USE THIS WORD UNLESS YOU ABSOLUTELY HAVE TO !
  >R R@ PICK
  SP@ DUP 2+   R> 1+ 2* CMOVE>  DROP  ;


\ c@ @ 2@                                          sfp 02/01/89

CODE C@         \ addr -- b
  ldbze tos tos []
  next,   end-code

CODE @          \ addr -- n
  ld tos tos []    next,   end-code

CODE 2@         \ addr -- d
  ld ax 2 tos ,[]   push ax
  ld tos tos []
  next,   end-code


\ c! ! 2! +!

CODE C!         \ b addr --
  pop ax   stb al tos []   pop tos
  next,   end-code

CODE !          \ n addr --
  pop tos []   pop tos
  next,   end-code

CODE 2!         \ d addr --
  pop tos []   pop 2 tos ,[]
  pop tos   next,   end-code

CODE +!         \ n addr --
  pop ax   add ax tos []   st ax tos []
  pop tos   next,   end-code


\ 1+ 2+ 1- 2- 2* 2/                                sfp 02/01/89

CODE 1+         \ n -- n+1
  inc tos   next,   end-code
CODE 2+         \ n -- n+2
  add tos # 2   next,   end-code

code 1-         \ n -- n-1
  dec tos   next,   end-code
code 2-         \ n -- n-2
  sub tos # 2   next,   end-code

CODE 2*         \ n1 -- n2
  shl tos # 1   next,   end-code
CODE 2/         \ n1 -- n2
  shra tos # 1   next,   end-code
CODE u2/	\ n1 -- n2
  shr tos # 1   next,   end-code

\ d< d>

CODE D<         \ d1 d2 -- t/f
  pop tosext
  cmp tos sp []+   jgt d<tru   jlt d<fls
  cmp tosext sp []  jh d<tru
l: d<fls
  add sp # 2   clr tos   next,
end-code

code d>         \ d1 d2 -- t/f
  pop tosext
  cmp tos sp []+   jlt d<tru   jgt d<fls
  cmp tosext sp []   jh d<fls   je d<fls	\ JL d<tru - no JL instruction
l: d<tru
  add sp # 2   ld tos # -1   next,
end-code


\ d= d0=                                           sfp 02/01/89

code d=         \ d1 d2 -- t/f
  pop tosext
  cmp tos sp []+   jne d<fls            \ false if different
  cmp tosext sp []   jne d<fls          \ false if different
  sjmp d<tru                            \ true if same!
end-code

code d0=        \ d -- t/f
  or tos sp []+
  ne, if,   clr tos   next,   endif,
  ld tos # -1   next,   end-code

\ within?                                          sfp 02/01/89

code within?    \ x a b -- t/f ; true if a<=x<=b
  pop tosext   pop ax
  cmp ax tosext   jlt wit1
  cmp ax tos   jgt wit1
  ld tos # -1   next,
l: wit1
  clr tos   next,   end-code

\ here allot c, ,                                  sfp 10/11/88

: here          \ -- addr
  dp @  ;

: there         \ -- addr
  dp @  ;

: allot         \ n --
  dp +!  ;

: c,            \ b --
  here c!  1 allot  ;

: ,             \ n --
  here !  2 allot  ;

\ on off count bounds                              sfp 11/01/89

code on         \ addr --
  ld ax # -1   st ax tos []   pop tos   next,   end-code

code off        \ addr --
  st r0 tos []   pop tos   next,   end-code

code count      \ addr -- addr+1 len
  clrb ah   ldb al tos []+
  push tos   ld tos ax   next,   end-code

code bounds     \ addr len -- addr+len addr
  ld ax sp []   add tos ax   st tos sp []   ld tos ax
  next,   end-code



\ upc upper

code upc        \ char -- char' ; enforce upper case
  cmpb tos.l # ascii a   jlt upcx
  cmpb tos.l # ascii z   jgt upcx
  andb tos.l # 0DF
l: upcx
  next,   end-code

code upper      \ addr len -- ; convert string to upper case
  pop bx   cmp tos r0    je upperx
l: upper1
  ldb al bx []+
  cmpb al # ascii a   jlt upper2
  cmpb al # ascii z   jgt upper2
  andb al # 0DF   stb al -1 bx ,[]
l: upper2
  dec tos   jne upper1
l: upperx
  pop tos   next,   end-code


\ toggle-bit set-bit reset-bit test-bit            sfp 02/01/89

code toggle-bit         \ mask addr --
  pop ax   xorb al tos []   stb al tos []
  pop tos   next,   end-code

code set-bit            \ mask addr
  pop ax   orb al tos []   stb al tos []
  pop tos   next,   end-code

code reset-bit          \ mask addr
  pop ax   notb al   andb al tos []   stb al tos []
  pop tos   next,   end-code

code test-bit           \ mask addr -- n
  pop ax   andb tos.l al & tos []   clrb tos.h
  next,   end-code
\ name> >name                                      sfp 02/01/89

hex

code name>                              \ nfa -- cfa
  ldbze ax tos []+   andb al # 01F      \ ax := count
  add tos ax   and ax tos & # 01        \ see if even
  e, if,  inc tos   endif,              \ make cfa odd
  next,   end-code                      \

code >name                              \ cfa -- nfa
l: to-name-1                            \
  dec tos   ldb al tos []               \ get byte
  cmpb al r0   jge to-name-1            \ until top bit set
  next,   end-code


\ skip scan

code skip       \ addr len char -- addr' len'
  pop cx   pop bx                       \ get parameters
  cmp cx r0   je skipx                  \ check length <> 0
l: skip1
  cmpb tos.l bx []+   jne skip2         \ compare characters
  dec cx   jne skip1                    \ until all done
l: skipx
  bx push   ld tos cx   next,           \ return addr & len
l: skip2
  bx dec   sjmp skipx   end-code        \ backup one

code scan       \ addr len char -- addr' len'
  pop cx   pop bx                       \ get parameters
  cmp cx r0   je skipx                  \ check length <> 0
l: scan1
  cmpb tos.l bx []+   je skip2          \ compare characters
  dec cx   jne scan1                    \ until all done
  sjmp skipx   end-code                 \

\ v-find ..........................................sfp 04/02/89

code v-find     \ text-addr voc-addr -- cfa 1 | cfa -1 | text 0
  cmp tos r0   je pf-fail               \ get voc, exists?
  ld tos 3 tos ,[]                      \ get addr of voc. data
                \ hash function is count+byte1
  ld bx sp []                           \ fetch text address
  ldbze ax bx []   addb al 1 bx ,[]     \ count + first byte
  ld cx tos []   dec cx                 \ put #threads-1
  and ax cx   shl ax # 1                \ range 0..#t-1, *2
  add tos ax                            \ add to base address
  ld tos 2 tos ,[]                      \ get top nfa
  cmp tos r0   je pf-fail               \ whoops, empty thread

\ tos=NFA [sp]=text
l: pf-loop
  ld ax sp []                           \ get text address
  ld bx tos                             \ get NFA
  ldb cl ax []+   ldb ch cl             \ get text count byte
  xorb ch bx []+                        \ XOR with NFA count
  andb ch # 03F  jne pf-retry           \ not same, failed

l: pf-scan
  ldb dl ax []+   cmpb dl bx []+        \ compare bytes
  jne pf-retry                          \ not same
  djnz cl pf-scan                       \ until done
\ fall through to pf-pass
l: pf-pass
  and ax bx & # 1                       \ if CFA even
  e, if,  inc bx   endif,               \ make it odd
  st bx sp []                           \  and save CFA
  ld ax # -1                            \ assume non-immediate
  ldb dl # 040   andb dl tos []         \ test immediate bit
  ne, if,  add ax # 2  endif,           \ return 1 if immediate
  ld tos ax   next,                     \ return flag, done

l: pf-retry
  ld tos -2 tos ,[]                     \ get next NFA
  cmp tos r0   jne pf-loop              \ check if end (NFA=0)

l: pf-fail
  clr tos				\ return 0=fail
  next,   end-code


\ call                                             sfp 04/02/89

code call       \ addr ; call assembler routine
  push # call-ret                       \ push return address
  br tos []                             \ jump to routine
l: call-ret                             \ get back here
  pop tos                               \ restore Forth stack
  next,   end-code

\ : variable                                       sfp 02/01/89

TARGET-ONLY
: :
  ?EXEC !CSP  CURRENT @ CONTEXT !
  CREATE  smudge  [compile] ]  ;CODE
  sub rp # 2   st ip rp []              \ save IP on R. stack
  pop ip                                \ get new IP
  next,   end-code   IMMEDIATE          \ go to it
HOST&TARGET

: VARIABLE
  CREATE  here 2+ ,  0 ,  ;CODE
  pop ax   push tos   ld tos ax []
  next,   end-code


\ constant user .........................          sfp 02/01/89

: CONSTANT
  CREATE  ,  ;CODE
  pop ax   push tos   ld tos ax []
  next,   end-code

: USER
  CREATE  ,  ;CODE
  pop ax   push tos   add tos up & ax []
  next,   end-code


\ (;code) does>                                    sfp 11/01/89

HEX

: (;CODE)       \ -- ; set return address as dest of CFA code
  latest name> 1+                       \ addr of dest
  r> over 2+ -                          \ jmp offset
  swap !  ;                             \ stored

: DOES>                                 \ --
  COMPILE (;CODE)                       \ end of compile action
  0FD C,  0EF C,  DODOES here 2+ - ,  ; \ NOP  LCALL DODOES
                IMMEDIATE
internal
: docreate,                             \ -- ; lay cfa
  0EF C,  DOCREATE here 2+ - ,  ;       \ lay down m/c call
external

\ defer crash                                      sfp 12/01/89

HEX

internal
: CRASH
  0C ERROR  ;
external

: DEFER
  CREATE  here 2+ ,  ['] CRASH ,  ;CODE
  pop ax   ld bx ax []                  \ address of data in RAM
  ld ax bx []                           \ get CFA
  br ax []   end-code                   \ go do it

