\ Code for Hitachi LMG6400PLGR LCD Display driver and 4x4 Matrix Keyboard Demo

\ Copyright (C) 1992
\ MicroProcessor Engineering Ltd
\ 133 Hill Lane
\ Southampton SO1 5AF 
\ England

\ Phone	 +44 703 631441
\ Fax	 +44 703 339691
\ E-mail mpe@cix.compulink.co.uk

\ constants and variables

2 constant cell		\ how big is a cell? Used in stack check code

\ application words
hex

\ -- ; actions performed by executing from the key action table
: emit-0 	ascii 0 lcd-emit ;
: emit-1 	ascii 1 lcd-emit ;
: emit-2 	ascii 2 lcd-emit ;
: emit-3 	ascii 3 lcd-emit ;
: emit-4 	ascii 4 lcd-emit ;
: emit-5 	ascii 5 lcd-emit ;
: emit-6 	ascii 6 lcd-emit ;
: emit-7 	ascii 7 lcd-emit ;
: emit-8	ascii 8 lcd-emit ;
: emit-9	ascii 9 lcd-emit ;

: emit-crlf	cr-char lcd-emit  lf-char lcd-emit  ;	      

\ application keypad assignment

hex
	  
: kb-init	\ -- ; perform the key action assignments
  assign emit-7        	00 to-key
  assign emit-8        	01 to-key
  assign emit-9        	02 to-key
  assign move-up       	03 to-key
			
  assign emit-4		04 to-key
  assign emit-5 	05 to-key
  assign emit-6 	06 to-key
  assign move-down	07 to-key

  assign emit-1 	08 to-key
  assign emit-2 	09 to-key
  assign emit-3 	0A to-key
  assign move-left	0B to-key

  assign lcd-clear	0C to-key
  assign emit-0 	0D to-key
  assign emit-crlf	0E to-key
  assign move-right    	0F to-key

  assign noop		#keys to-key	\ action for a duff key
;


\ keyboard polling routine

: (stack-check)		\ -- ; complain if either stack depth is non zero
   depth			\ diagnostic code 
   if  single
       cr ." Internal stack error in keyboard poller :-" depth .
       abort
   endif
   rp@ r0 @ cell - <>		\ there should be 1 item on the r-stack
   if  single
       cr ." Return stack fault in keyboard poller"
       abort
   endif		       
;

defer stack-check

((
This stack check is deferred. This is because if you wish to implement
something which needs to make use of the stack, eg a simple calculator
the stack check will normally trap it as an error. However if this is
not required then the stack check ought to be made.

Pick one of the two assignments below
))

  assign (stack-check)	to-do stack-check 	\ perform stack checking
\ assign noop 		to-do stack-check	\ no stack checking


((
Polling routine which continually reads each line
and then executes the correct word depending on the key pressed
))     

: poll-kb	\ -- 
  r> drop			\ will not exit
  rp@ r0 !  sp@ s0 !		\ find stack initial values
  begin 					
    #keys inkey !		\ reset the inkey so we can tell when done	  
    keypad 			\ read keypad
    do-key			\ perform action if key pressed
    stack-check			\ diagnostic check
    pause
  again 
;

\ Initialisation word

\ Set everything up in one go
: init-demo		\ --
  pia-init			\ initialise the PIA for 2.5 o/p 0.5 i/p
  lcd-init			\ set up the display
  kb-init			\ set-up the default key assignments
;

\ multi-task it

: run-demo	\ --
  init-demo			\ initialise the hardware
  both				\ make the forth display on the lcd
  init-multi  multi		\ initialise multitasker
  assign poll-kb 1 to-task	\ set up the application on a task
  1 activate			\ activate the application
  lcd-clear			\ clear screen
;

