\ Train controller for Racetech 80C196 board

pto

Delay values adjusted for 16MHz MPE PowerBoard

Copyright (c) 1989..1992
MicroProcessor Engineering
133 Hill Lane
Southampton SO1 5AF
tel: (+44) 703 631441
fax: (+44) 703 339691
net: mpe@cix.compulink.co.uk

This code drives a simple motor controller that needs a pulse
every 20ms. The pulse width is supposed to be as as follows:
  1 ms        full reverse
  1.5 ms      stop (small dead band)
  2 ms        full forward
			  
\ Application code

decimal

variable msdelay  170 msdelay !		\ delay for 1 ms
variable rundelay  240 rundelay !	\ used for speed controller
variable stopdelay  170 stopdelay !	\ delay time for stopped

13 constant Enter                      \ ASCII code for Enter key

: 1ms			\ -- ; wait 1 millisecond
  msdelay @ 0 ?do loop
;

: ms			\ n -- ; wait n milliseconds
  0 ?do  1ms  loop
;

hex

: run-pulse		\ -- ; pulse o/p high 
  0E000 on				\ pulse high output port at 0E000h
  rundelay @ 0 ?do  loop		\ wait required period
  0E000 off				\ pulse low
;

decimal

: cycle			\ -- ; run one cycle for the controller
  run-pulse  19 ms
;

: cycles		\ -- ; perform n cycles
  0
  ?do  cycle  loop  
;

: run			\ -- ; run until key pressed
  begin  cycle  key? until  
;

: speed			\ n -- ; run with given speed rating
  rundelay !  run  
;

\ -----End of Page-----
\ -- R.G.G -- 3/12/91 --  Changing speed

: ascii			\ -- char ; return 1st character of following token
  bl word 1+ c@				\ collect character
  state @				\ if compiling
  if  [compile] literal  endif		\   lay literal
; immediate

: .speed		\ -- ; display current speed 
  cr rundelay @ u.  
;

: stop			\ -- ; stop it
  stopdelay @ rundelay !		\ set current speed to stopped
  .speed				\ display it
;

: +speed		\ n -- ; towards full forward
  rundelay @ +			\ update speed
  0 max  stopdelay @ 2* min	\ range check it
  rundelay !			\ store it
  .speed			\ display it
;

: special-key?		\ -- t/f ; true if exit key
  case  key upc
    ascii F  of  -1 +speed  0 endof
    ascii R  of  1 +speed   0 endof
    ascii U  of  -1 +speed  0 endof
    ascii D  of  1 +speed   0 endof
    ascii S  of  stop       0 endof
    Enter    of            -1 endof
             0 swap
  endcase
;

\ -----End of Page-----
\ -- R.G.G -- 3/12/91 --  controlling speed

: decelerate?		\ -- t/f ; true if not stopped
  stopdelay @ rundelay @ -	\ difference between current and stopped
  dup -1 max  1 min		\ force to range -1, 0, +1
  +speed			\ update speed accordingly
;

: decelerate		\ -- ; cause a slow decelerate
  begin
    decelerate?			\ deceleration needed ?
   while
    10 cycles			\ yes, run a bit at new speed
  repeat			\ go try again
;

: control		\ -- ; control train
  stop				\ initialise to stopped condition
  begin  run			\ then run
         special-key?		\ until the exit key is pressed
  until  
  decelerate  			\ finally, decelerate to a halt
;

