\ 8096/80C196 ROM PowerForth - Interrupt handlers

pto

Copyright (c) 1988..1991 to:

MicroProcessor Engineering Limited
133 Hill Lane
Shirley
Southampton SO1 5AF
England

tel: (+44) 703 631441   ( international )
tel: 0703 631441        ( domestic )

Oct 1991	Separated from multitasker code


\ normal register equates

\ 0   constant r0                 \ register 0 always returns 0
\ 018 constant SP                 \ data stack pointer
\ 01A constant RP                 \ return stack pointer
\ PLM style register set - allows mixed language programming
\ 01C constant AX         01C constant AL   01D constant AH
\ 01E constant BX         01E constant BL   01F constant BH
\ 020 constant CX         020 constant CL   021 constant CH
\ 022 constant DX         022 constant DL   023 constant DH
\ 024 constant IP                 \ Forth Interpretive pointer
\ 026 constant UP                 \ user area pointer
\ 028 constant TOS                \ top of data stack register
\   028 constant tos.l            \ NB TOS is LONG aligned
\   029 constant tos.h
\ 02A constant tosext             \ TOS extension

\ ei di save-int restore-int                       sfp 12/01/89

code ei                 \ -- ; enable interrupts
  ei   next,   end-code

code di                 \ -- ; disable interrupts
  di   next,   end-code

code save-int           \ -- n ; save interrupt status
  push tos   pushf   pop tos   di       \ save int, turn it off
  next,   end-code                      \

code restore-int        \ n -- ; restore interrupt
  push tos   popf   pop tos   next,   end-code

\ reti - return from interrupt                     sfp 05/02/89

code reti       \ return from high level interrupt
l: [hreti]
  pop TOSEXT    pop TOS                 \ stack top registers
  pop UP                                \ user area pointer
  pop DX   pop CX   pop BX   pop AX     \ scratch registers
  pop ip   popa   ret                   \ IP, flags, done
end-code


\ next-int - common interrupt handler              sfp 05/02/89

assembler
l: next-int     \ save context and restart Forth
  push AX   push BX			\ PLM register set - part 1
  push CX   push DX			\ PLM register set - part 2
					\ IP already done
  push UP				\ user area pointer
  push TOS    push TOSEXT		\ top of data stack register(s)

  ld up # int-init-u0			\ interrupt user area
					\ IP already set
  next,
forth

\ Timer overflow interrupt                         sfp 05/02/89
((
This code provides an example of a high level interrupt handler.
The high level Forth ISR word is deferred so that the user may
reassign the action at will.
))

HEX

variable tov-ticks                      \ counter

: tov-count     \ -- ; increment ticks counter
  1 tov-ticks +!  ;

: start-tov     \ -- ; setup and enable overflow interrupts
  1 int_mask c!				\ enable TOV interrupts
  004 ioc1 set-bit			\ enable TOV in IOC1
  ei  ;                                 \ enable any interrupts

defer tov-isr				\ DEFERred word for each interrupt
  assign tov-count to-do tov-isr

l: tov-int
 ] tov-isr reti [			\ action, return

ASSEMBLER                               \
l: tov-int0                             \ registers saved by IRQ
  pusha   push ip                       \ save PSW and IP
  ld ip # tov-int   ljmp next-int       \ load new IP
FORTH
tov-int0 02000 !                        \ set vector


defer a/d-isr				\ DEFERred word for each interrupt
((
: a/d-def	\ --
  ." A/D int. not initialised"  ;
assign a/d-def to-do a/d-isr
))

l: a/d-int
 ] a/d-isr reti [			\ action, return

ASSEMBLER
l: a/d-int0				\ registers saved by IRQ
  pusha   push ip                       \ save PSW and IP
  ld ip # a/d-int   ljmp next-int	\ load new IP
FORTH
a/d-int0 02002 !			\ set vector

\ hsi-isr hso-isr hsi.0-isr

defer hsi-isr				\ DEFERred word for each interrupt
((
: hsi-def	\ --
  ." HSI int. not initialised"  ;
assign hsi-def to-do hsi-isr
))

l: hsi-int
 ] hsi-isr reti [			\ action, return

ASSEMBLER
l: hsi-int0				\ registers saved by IRQ
  pusha   push ip                       \ save PSW and IP
  ld ip # hsi-int   ljmp next-int	\ load new IP
FORTH
hsi-int0 02004 !			\ set vector


defer hso-isr				\ DEFERred word for each interrupt
((
: hso-def	\ --
  ." HSO int. not initialised"  ;
assign hso-def to-do hso-isr
))

l: hso-int
 ] hso-isr reti [			\ action, return

ASSEMBLER
l: hso-int0				\ registers saved by IRQ
  pusha   push ip                       \ save PSW and IP
  ld ip # hso-int   ljmp next-int	\ load new IP
FORTH
hso-int0 02006 !			\ set vector


defer hsi.0-isr				\ DEFERred word for each interrupt
((
: hsi.0-def	\ --
  ." HSI.0 int. not initialised"  ;
assign hsi.0-def to-do hsi.0-isr
))
l: hsi.0-int
 ] hsi.0-isr reti [			\ action, return

ASSEMBLER
l: hsi.0-int0				\ registers saved by IRQ
  pusha   push ip                       \ save PSW and IP
  ld ip # hsi.0-int   ljmp next-int	\ load new IP
FORTH
hsi.0-int0 02008 !			\ set vector

\ timer-isr ext-isr trap-isr illop-isr

defer timer-isr				\ DEFERred word for each interrupt
((
: timer-def	\ --
  ." TIMER int. not initialised"  ;
assign timer-def to-do timer-isr
))

l: timer-int
 ] timer-isr reti [			\ action, return

ASSEMBLER
l: timer-int0				\ registers saved by IRQ
  pusha   push ip                       \ save PSW and IP
  ld ip # timer-int   ljmp next-int	\ load new IP
FORTH
timer-int0 0200A !			\ set vector


defer ext-isr				\ DEFERred word for each interrupt
((
: ext-def	\ --
  ." EXT int. not initialised"  ;
assign ext-def to-do ext-isr
))

l: ext-int
 ] ext-isr reti [			\ action, return

ASSEMBLER
l: ext-int0				\ registers saved by IRQ
  pusha   push ip                       \ save PSW and IP
  ld ip # ext-int   ljmp next-int	\ load new IP
FORTH
ext-int0 0200E !			\ set vector


defer trap-isr				\ DEFERred word for each interrupt
((
: trap-def	\ --
  ." TRAP int. not initialised"  ;
assign trap-def to-do trap-isr
))

l: trap-int
 ] trap-isr reti [			\ action, return

ASSEMBLER
l: trap-int0				\ registers saved by IRQ
  pusha   push ip                       \ save PSW and IP
  ld ip # trap-int   ljmp next-int	\ load new IP
FORTH
trap-int0 02010 !			\ set vector


defer illop-isr				\ DEFERred word for each interrupt
((
: illop-def	\ --
  ." ILLOP int. not initialised"  ;
assign illop-def to-do illop-isr
))

l: illop-int
 ] illop-isr reti [			\ action, return

ASSEMBLER
l: illop-int0				\ registers saved by IRQ
  pusha   push ip                       \ save PSW and IP
  ld ip # illop-int   ljmp next-int	\ load new IP
FORTH
illop-int0 02012 !			\ set vector

\ hsi4-isr t2c-isr t2o-isr ext1-isr

defer hsi4-isr			\ DEFERred word for each interrupt
((
: hsi4-def	\ --
  ." HSI4 int. not initialised"  ;
assign hsi4-def to-do hsi4-isr
))

l: hsi4-int
 ] hsi4-isr reti [			\ action, return

ASSEMBLER
l: hsi4-int0				\ registers saved by IRQ
  pusha   push ip                       \ save PSW and IP
  ld ip # hsi4-int   ljmp next-int	\ load new IP
FORTH
hsi4-int0 02034 !			\ set vector


defer t2c-isr				\ DEFERred word for each interrupt
((
: t2c-def	\ --
  ." T2C int. not initialised"  ;
assign t2c-def to-do t2c-isr
))

l: t2c-int
 ] t2c-isr reti [			\ action, return

ASSEMBLER
l: t2c-int0				\ registers saved by IRQ
  pusha   push ip                       \ save PSW and IP
  ld ip # t2c-int   ljmp next-int	\ load new IP
FORTH
t2c-int0 02036 !			\ set vector


defer t2o-isr				\ DEFERred word for each interrupt
((
: t2o-def	\ --
  ." T2O int. not initialised"  ;
assign t2o-def to-do t2o-isr
))

l: t2o-int
 ] t2o-isr reti [			\ action, return

ASSEMBLER
l: t2o-int0				\ registers saved by IRQ
  pusha   push ip                       \ save PSW and IP
  ld ip # t2o-int   ljmp next-int	\ load new IP
FORTH
t2o-int0 02038 !			\ set vector


defer ext1-isr				\ DEFERred word for each interrupt
((
: ext1-def	\ --
  ." EXT1 int. not initialised"  ;
assign ext1-def to-do ext1-isr
))

l: ext1-int
 ] ext1-isr reti [			\ action, return

ASSEMBLER
l: ext1-int0				\ registers saved by IRQ
  pusha   push ip                       \ save PSW and IP
  ld ip # ext1-int   ljmp next-int	\ load new IP
FORTH
ext1-int0 0203A !			\ set vector

\ hsifull-isr nmi-isr

defer hsifull-isr			\ DEFERred word for each interrupt
((
: hsifull-def	\ --
  ." HSI FULL int. not initialised"  ;
assign hsifull-def to-do hsifull-isr
))

l: hsifull-int
 ] hsifull-isr reti [			\ action, return

ASSEMBLER
l: hsifull-int0				\ registers saved by IRQ
  pusha   push ip                       \ save PSW and IP
  ld ip # hsifull-int   ljmp next-int	\ load new IP
FORTH
hsifull-int0 0203C !			\ set vector


defer nmi-isr				\ DEFERred word for each interrupt
((
: nmi-def	\ --
  ." NMI int. not initialised"  ;
assign nmi-def to-do nmi-isr
))

l: nmi-int
 ] nmi-isr reti [			\ action, return

ASSEMBLER
l: nmi-int0				\ registers saved by IRQ
  pusha   push ip                       \ save PSW and IP
  ld ip # nmi-int   ljmp next-int	\ load new IP
FORTH
nmi-int0 0203E !			\ set vector
