\ MPE 80C196 ROM PowerForth Forth kernel - high level code

pto

released for use without royalty when used with the
MPE 809x/80C19x Forth Cross Compiler.

Copyright (c) 1987-1992

MicroProcessor Engineering
133 Hill Lane
Shirley
Southampton SO1 5AF
England

tel: (+44) 703 631441	(international)
     0703 631441	(domestic)
fax: (+44) 703 339691
net: mpe@cix.compulink.co.uk

May 1991	Converted to text file

\ system equates ................................. sfp 10/05/91

hex

\ character equates
020 EQU ABL
00D EQU ACR
02E EQU ADOT
007 EQU ABELL
008 EQU BSIN
008 EQU BSOUT
00A EQU ALF
00C EQU FFEED

\ true and false
000 EQU FALSE
 -1 EQU TRUE
\ start-up values                                  sfp 26/01/90

\
\ Initial values of user variables are held as a table in the same order
\ as the user variables themselves are defined. At cold start code in (INIT)
\ copies this table from ROM into the USER area.
\
L: USER-RESET
  init-s0 ,   init-r0 ,         \ s0, r0
  0 ,  0 ,                      \ #tib, span
  0 ,  0 ,                      \ >in, out
  0A ,  0 ,                     \ base, hld

\
\ The cross compiler will fill in these locations with the initial
\ values of FENCE, DP, and VOC-LINK. At cold start, code in (INIT) copies
\ these values to the variable FENCE, DP, and VOC-LINK.
\
L: INIT-FENCE			\ fence
  0 ,
L: INIT-DP			\ dp
  0 ,
L: INIT-VOC-LINK		\ voc-link
  0 ,

\ ; ...............................................sfp 03/10/89

HEX

: ;		\ -- ; -- [child] ; addr r-- [child] ; finishes colon word
  ?comp ?csp				\ error checks
  compile exit				\ EXIT will leave child
  smudge				\ make header available
  [compile] [  ;   immediate		\ back to INTERPRET mode

\ Vocabulary structure ..................          Jon 19/08/88

((
                  |------------->------------|
                  |                          |
| nfa | cfa | ^threads | voc-link | .. | #threads | threads .. |
      ^
Context, Current                       This part in RAM on
and Voc-Link                           cross compiled
point to the CFA                       vocabularies for
                                       ROM / RAM version.
))

: Vocabulary	\ -- ; -- [child] ; create new vocabulary
  Create
    here dup 4 + ,                      \ lay pointer to threads
    Voc-Link @ , body>  Voc-Link !      \ update voc-link
    #threads @ dup ,                    \ lay number of threads
    0 ?do 0 , loop                      \ zero allot threads
  Does>
    body> Context !                     \ CFA in context
;
 
Vocabulary Forth			\ primary Forth vocabulary
Vocabulary Root				\ stub for escape routes

only forth definitions			\ restating the obvious

\
\ Dictionary structure navigation
\
\ All fields within a dictionary header or vocabulary structure are
\ referenced from the code field addess (CFA).
\
decimal

: n>link	\ nfa -- lfa ; step from name field to link field
  2-   ; 
: link>n	\ lfa -- nfa ; step from link field to name field
  2+   ;

: >Body		\ cfa -- pfa ; step from code field to parameter field
  3 +  ;
: Body>		\ pfa -- cfa ; step from parameter field to code field
  3 -  ;

: >Voc-Link	\ cfa -- v-lfa ; step from code field to voc-link field
  >body 2+  ;
: Voc-Link>	\ v-lfa -- cfa ; step from voc-link field to code field
  2- body>  ;

: >#Threads	\ cfa -- #threads ; step from code field to #threads field
  >body @  ;
: >Threads	\ cfa -- threads ; step from code field to threads field
  >body @ 2+  ;				\ no way from threads to cfa.

\ user variables                                   sfp 26/01/90

hex

000 user S0		\ data stack pointer reset value
002 user R0		\ return stack pointer reset value
004 user #TIB		\ number of characters in input buffer
006 user SPAN		\ number of characters collected by EXPECT
008 user >IN		\ text interpreter's position in input buffer
00A user OUT		\ number of characters output on current output line
00C user BASE		\ number base used for text/number conversion
00E user HLD		\ pointer to number output conversion buffer
010 user #L		\ size of converted number, 0=fail, 1=single, 2=double
012 user #D		\ total number of digits converted
014 user DPL		\ number of characters after double point
			\ initialised to -1, set to 0 by ',', incr. thereon
016 user 'TIB		\ points to terminal input buffer

040 user PAD		\ string scratch area, preceded by at least 34 
			\ characters for the number output conversion buffer
			\ 32 characters for double number + sign, BASE=2
			\ then allow 32 characters for PAD


\ system variables                                 sfp 26/01/90

variable WIDTH  01f width !             \ max name width
variable FENCE                          \ protected dictionary
variable DP                             \ dictionary pointer
variable VOC-LINK                       \ links vocabs.
variable BLK                            \ for mass storage
variable SCR                            \ for mass storage
variable CURRENT                        \ vocabulary in which to put words
variable STATE                          \ interpreting or compiling
variable CSP                            \ saved stack pointer for error checks
08 equ #VOCS				\ number of vocs. in search order
#VOCS 2 * EQU VSIZE			\ size of array to hold them
variable CONTEXT  vsize 2- allot-ram	\ 8 vocabulary search order array
variable LAST                           \ points to last NFA
variable #THREADS   010 #threads !      \ #threads in new voc.
variable XON/XOFF                       \ true if in use
variable ECHOING    -1 echoing !        \ true if echoing

\ deferred system words
defer error             \ main error handler
defer block             \ traditional i/o handler

\ constants                                        sfp 26/01/90
hex
  0 CONSTANT 0
  1 CONSTANT 1
  2 CONSTANT 2
 -1 CONSTANT -1
020 CONSTANT BL
040 CONSTANT C/L
011 constant xon
013 constant xoff
\ latest smudge immediate ?error !csp ?csp ?comp ?exec ?stack tib compile

hex

: latest	\ -- addr ; return NFA of last word defined
  last @ ;

: smudge	\ -- ; toggle HIDE/REVEAL bit in name field
  020 latest toggle-bit  ;

: immediate	\ -- ; set IMMEDIATE bit in name field
  040 latest set-bit  ;

: ?error	\ flag n -- ; run n ERROR if flag is non-zero (true)
  swap
  if  error  else  drop  endif  ;

: !csp		\ -- ; save current stack pointer for later error check
  sp@ csp ! ;

: ?csp		\ -- ; error if stack depths don't match
  sp@ csp @ <> 014 ?error ;

: ?comp		\ -- ; error if not compiling
  state @ 0= 011 ?error ;

: ?exec		\ -- ; error if not in INTERPRET mode
  state @ 012 ?error ;

: ?stack	\ -- ; error if stack underflowed
  sp@ s0 @ u> 1 ?error  ;

: tib		\ -- addr ; return address of terminal input buffer 
  'tib @  ;

: compile	\ -- ; compile CFA of word following in-line
  ?comp  r> dup 2+ >r  @ ,  ;


\ only also previous definitions forth

also root definitions
: only		\ -- ; search order becomes only ROOT
  ['] root				\ will be stored at end of CONTEXT
  context [ vsize 2- ] literal		\ length to clear
  2dup erase  + !  			\ clear, put ROOT at end
  root  ;

: also		\ -- ; shuffle array up one, leaving last intact
  context dup 2+			\ shuffle by copy
  [ vsize 4 - ] literal cmove>  ;

: previous	\ -- ; shuffle array down one
  context 2+  context			\ source and destination addresses
  [ vsize 4 - ] literal cmove		\ length and copy
  [ context vsize 4 - + ] literal off ;	\ clear vacated slot

: definitions	\ -- ; set voc. into which words are put
  context @ current !  ;

' forth equ oldforth			\ kluge coming
: forth		\ -- ; need a version of FORTH in the FORTH vocabulary
  [ oldforth , ]  ;			\ avoid self reference

: seal		\ -- ; delete all but first vocabulary
  context 2+ vsize 2- erase  ;
only forth definitions
\ even align

: even          \ addr -- addr' ; make number even
  dup 1 and                             \ odd ?
  if  1+  endif  ;                      \ yes, step on 1

: align         \ -- ; force dictionary to even alignment
  here 1 and                            \ dictionary odd ?
  if  BL c,  endif  ;                   \ insert space char


\ $. (.") (abort") (")

: $.		\ $addr -- ; display string
  count type  ;

: (.")		\ -- ; display in-line string and skip over it
  r> count 2dup + even >r   type  ;

: (abort")	\ flag -- ; action of ABORT" which aborts if flag is set
  if  cr r@ $.  abort			\ set, display in-line string, abort
  else  r> count + even >r		\ not set, skip in-line string
  endif  ;

: (")		\ -- addr ; return address of in-line string, and skip it
  r> dup count + even >r  ;


\ pad #> hold sign # #s <# 

hex

: hold		\ char -- ; place character in number string below PAD
  -1 hld +!  hld @ c!  ;

: sign		\ n d -- d ; put sign of number in string
  rot 0<				\ n negative?
  if  ascii - hold  endif  ;		\ y: stash '-' in string

: #		\ d -- d' ; convert least significant digit of d to ASCII
  base @ mu/mod				\ divide d by current base
  rot 9 over <				\ convert remainder to ASCII 0..9,A..Z
  if  7 +  endif			\ allow for 9..A gap
  030 + hold  ;				\ add '0'

: #s		\ d -- d' ; convert digits until d'=0
  begin  # 2dup or  0= until  ;

: <#		\ -- ; initialise conversion buffer pointer HLD
  pad hld !  ;

: #>		\ d -- addr len ; finish conversion, return string addr/len
  2drop					\ discard converted number, now 0
  hld @  pad over -  ;			\ calculate string address and length
\ move

: move		\ addr1 addr2 count -- ; intelligent cmove
  >r					\ hide count value
  2dup u<				\ compare addresses, addr1<addr2 ?
  if  r> cmove>				\ y: move end first
  else  r> cmove			\ n: move start first
  endif  ;


\ space spaces d.r d. . u. .r u.r                  sfp 11/02/90

: space		\ -- ; display single space
  bl emit ;

: spaces	\ +n -- ; display n spaces if n positive
  0 max 0
  ?do  space  loop  ;

: d.r		\ d n -- ; display signed double in a field n chars wide
  >r					\ save field width
  swap over dabs <# #s sign #>		\ convert signed number
  r> over - spaces type ;		\ display result

: d.		\ d -- ; display d as signed number with trailing space
  0 d.r space ;

: .		\ n -- ; display n as signed number with trailing space
  s>d d.  ;

: u.		\ u -- ; display u as unsigned number with trailing space
  0 d.  ;

: .r		\ n1 n2 -- ; display signed n1 in a field n2 wide
  >r s>d r> d.r  ;

: u.r		\ u n -- ; dusplay unsigned u in a field n wide
  0 swap d.r  ;
\ hex decimal binary

hex

\ -- ; set BASE
: hex           010 base !  ;
: decimal       00A base !  ;
: binary	002 base !  ;


\ ascii control recurse


: ascii		\ -- n ; -- [compiling] ; ASCII ccc - return ASCII of 1st char
		\ or compile it as a literal if STATE is set
  bl word 1+ c@  state @		\ -- char [exec]
  if  [compile] literal	 endif  ;	\ -- [compiling]
  immediate				\ a rare STATE smart word

((
: control	\ -- n ; -- [compiling] ; CONTROL ccc - return ASCII of
		\ 1st char converted to CONTROL+char
		\ or compile it as a literal if STATE is set
  bl word 1+ c@  01F and		\ see ASCII
  state @
  if  [compile] literal  endif  ;
  immediate
))

: recurse	\ -- ; compile reference to last word defined,
		\ used inside colon definition permits direct recursion
  latest name> ,  ;  immediate 


\ erase blank

hex

: erase		\ addr cnt -- ; fill area with zero bytes
  0 fill  ;

: blank		\ addr cnt -- ; fill area with ASCII spaces
  bl fill  ;


\ abort (error) - error handlers

: abort		\ ?? -- ; reset data stack, run text interpreter
  s0 @ sp!  quit  ;

: (error)	\ n -- ; ERROR is deferred, this the default
		\ ERROR displays a message, and then runs ABORT
  base @ swap	decimal			\ preserve current base, messages in decimal
  echoing on				\ re enable echoing
  xon/xoff off				\ turn off flow control
  cr ." error # " .			\ show error number
  space here $. cr			\ show word causing error
  base !				\ restore base
  abort  ;				\ abort system

assign (error) to-do error		\ set up default error handler

\ input numbers - skip-sign +digit +char +ascii-digit (integer?) number?

internal
: skip-sign	\ addr1 len1 -- addr2 len2 t/f ; true if sign=negative
  over c@  dup ascii + =		\ first character '+'
  swap ascii - =  dup >r or		\ or '-'
  if  1 /string  endif			\ if so, skip it
  r>  ;					\ get sign back

: +digit	\ d1 n -- d2 ; accumulates digit into double accumulator
  swap base @ um* drop			\ accumulate n into double d1
  rot base @ um*  d+
  1 #d +!  dpl @ 1+			\ incr. #digits, double number?
  if  1 dpl +!  endif ;			\ yes, incr. #digits after d.p.

: +char		\ char -- t/f ; process special chars like ',' ; true if ok
  ascii , = dup				\ not digit, try ","
  if  drop  dpl @ 0<			\ is "," see if allowed
      if  dpl off  2			\ yes, set up and flag
      else  0  endif			\ illegal, flag it
  endif
  dup #l !  ;

: +ascii-digit	\ d1 char -- d2 t/f ; true=ok ; accumulate character into d1
  dup base @ digit			\ digit?
  if  nip +digit  #l @			\ yes, accumulate into double
  else  +char				\ not digit, try ","
  endif  ;
external

: (integer?)	\ addr len -- d/n/- 2/1/0 ; convert string to number
  #d off  -1 dpl !  1 #l !		\ initialise flags
  skip-sign over			\ process sign, check length remaining
  if  >r				\ save sign of number
      0, 2swap  bounds			\ initial double number, string limits
      do  i c@ +ascii-digit		\ convert a character
          0= ?leave			\ finish if invalid
      loop
      r> ?dnegate			\ adjust for sign
      #d @ 0=  #l @ 0= or		\ conversion failed?
      if  #l off  2drop			\ y:, discard results
      else  #l @ 1 =			\ n:, single number?
            if  drop  endif		\     y:, convert to single
      endif
  else  drop 2drop  #l off		\ zero length string
  endif
  #l @  ;				\ return converted number size

: integer?	\ addr -- d/n/- 2/1/0 ; convert string to number
  count  ?dup				\ if numbers in character
  if  base @ >r				\ save current base
      case  over c@			\ look at first character
	ascii $ of  hex  1 /string      endof	\ $xxxx is HEX number
	ascii # of  decimal  1 /string  endof	\ #xxxx is DECIMAL number
	ascii % of  binary  1 /string	endof	\ %xxxx is binary number
     endcase
     (integer?)				\ convert number
     r> base !				\ restore base
  else  drop  #l off  0			\ zero length string
  endif  ;

Defer Number?
  Assign Integer? to-do Number?

\ #literal literal dliteral 

internal
: #literal	\ n1 .... nn n -- ; put in dictionary n1 first
  ?dup					\ if anything to do
  if  0 swap 1-				\ n1..nn 0 n-1 --
      do  i roll  compile lit  ,	\ compile n literals
      -1 +loop
  endif  ;
external

: literal	\ n --
  1 #literal  ;  immediate

: dliteral	\ d --
  2 #literal  ;  immediate


\ find 

: find		\ addr1 -- addr2 n ; find word in current search order
  dup count upper  dup c@		\ if non-zero length
  if  0  context vsize bounds		\ for each vocabulary in search order
      do  drop				\ drop 0 from last search
          i @ v-find  dup ?leave	\ search in this voc, leave if found
      2 +loop				\ step to next voc
  else  0				\ zero length string
  endif  ;

\ ?emit save-char bs ?bs expect

\
\ These words are used by EXPECT. They cope with the ECHOING flag
\ which controls if characters input should be echoed.
\
internal
: ?emit		\ char -- ; emit if echoing enabled
  echoing @				\ stash char, echo it?
  if  emit  else  drop  endif ;		\ emit it or discard it

: save-ch	\ char addr -- ; save as required
  over ?emit  c!  ;			\ store char, emit if necessary

: bs		\ -- ; backspace if output pointer non-zero
  out @ ?dup				\ if counter non-zero
  if  bsout emit  space  bsout emit	\ backup and overwrite
      1- out !				\ reduce OUT by 1
  endif  ;

: ?bs           \ pos -- pos' step ; perform BS if pos non-zero
  dup 0<>  echoing @ 0<>  and           \ pos non-zero and echo
  if  bs 1- -1                          \ perform backspace
  else  0                               \ no
  endif ;
external

hex

: expect        \ addr len -- ; read up to LEN chars into ADDR
  xon/xoff @				\ if flow control required
  if  xon emit  endif			\ enable server
  0 -rot bounds                         \ form address limits
  ?do  case key                         \ get character
         dup 01F >			\ not control character
             ?of i save-ch 1+ 1   endof \ 020..0FF
         008 of ?bs               endof \ BS
         009 of bl i save-ch 1+ 1 endof \ TAB, convert to space
         00A of bl ?emit  leave   endof \ LF, treat like CR
         00C of bl ?emit  leave   endof \ FF, treat like CR
         00d of bl ?emit  leave   endof \ CR
                i save-ch  1+ 1 0	\ stash char, echo it?
       endcase                          \
  +loop					\ and round again
  xon/xoff @				\ if flow control required
  if  xoff emit  endif			\ disable server
  span !  ;                             \ set length received
\ place source parse-word word

hex

internal
: place		\ addr1 len addr2 -- ; copies uncounted string to counted
  2dup c!  1+ swap move  ;

: source	\ -- addr len ; returns address and length of i/p buffer
  blk @ ?dup				\ BLK<>0 means input from screen
  if  block 0400			\ so return BLOCK buffer address
  else  tib #tib @			\ otherwise return TIB and length
  endif  ;

: parse-word	\ char -- addr len ; look for word delimited by char
  >r					\ save char
  source tuck  >in @ /string		\ skip over >IN chars
  r@ skip				\ skip leading delimiters
  over swap r> scan			\ look for trailing delimeter
  >r					\ save remaining length
  over -				\ length of word
  rot  r> dup 0<> + - >in !  ;		\ recalculate >IN
external

: word		\ char -- addr ; scan for word, placing it at HERE
  parse-word  here place  here		\ put word at HERE
  dup count + bl swap c!  ;		\ add space (Forth-83 standard)


\ query interpret                                  sfp 11/02/90

hex

: query         \ -- ; collect input line from keyboard to TIB
  tib 050 expect			\ get input
  span @ #tib !				\ reset length of TIB
  >in off  ;				\ reset input pointer

: interpret     \ -- ; interpret line of text in TIB
  begin  ?stack  bl word c@		\ while a token is available
    while  here find ?dup		\ is it a Forth word?
           if  0< state @ and		\ y: non-immediate and compiling?
               if  ,			\    y: compile it
	       else  execute		\    n: execute it
	       endif
           else  number?		\ n: must be number
		 ?dup 0= 0 ?error	\    error if not number
		 state @		\    compiling?
                 if  #literal		\    y: compile literal
		 else  drop		\    n: leave number on stack
		 endif
           endif
  repeat  ;
\ [ ] depth                                        sfp 31/01/90

: [		\ -- ; turn compiler off
  state off  ;  immediate

: ]		\ -- ; turn compiler on
  state on  ;

: depth		\ -- n ; return no. of items on stack before DEPTH
  s0 @ sp@ - 2- 2/  ;


\ quit                                             sfp 31/01/90

: quit          \ -- ; the main text interpreter loop
  blk off  [compile] [			\ keyboard input, interpret mode
  begin   r0 @ rp!  echoing @		\ no exit, new line if echoing
          if  cr  endif
          query interpret		\ get line and interpret it
          state @ 0=  echoing @ 0<> and	\ if interpreting and echoing
          if  ."  ok"  depth ?dup	\ prompt is 'ok' and depth
              if  ." -" .  endif	\ if non-zero
          endif
  again  ;


\ >mark >resolve <mark <resolve ?pairs             sfp 11/02/90

hex
\
\ These words define high level branches. They are used by the structure
\ words such as IF and AGAIN.
\
internal
: >mark		\ -- addr ; mark start of forward branch
  here  0 ,  ;

: >resolve	\ addr -- ; resolve target of forward branch
  here swap !  ;

: <mark		\ -- addr ; mark start (destination) of backward branch
  here  ;

: <resolve	\ addr ; resolve (at branch point) backward branch
  ,  ;

: ?pairs	\ n1 n2 -- ; error if n1<>n2 ; structures have different n
  <> 013 ?error  ;
external
\ do loop ?do +loop leave ?leave 

\
\ Structure words are all immediate. They either compile branches or mark addresses or both.
\ They also leave magic numbers used for error checking that identify the type
\ of structure being compiled.
\
: do		\ -- addr1 addr2 6
  compile (do) >mark			\ exit address compiled after (DO)
  <mark  6  ;  immediate		\ (LOOP) branches back

: ?do		\ -- addr1 addr2 6
  compile (?do) >mark			\ exit address compiled after (?DO)
  <mark  6  ;  immediate		\ (LOOP) branches back

: loop          \ addr1 addr2 6 --
  6 ?pairs				\ error check
  compile (loop) <resolve		\ resolve branch back
  >resolve  ;  immediate		\ resolve branch out

: +loop		\ addr1 addr2 6 --
  6 ?pairs				\ error check
  compile (+loop) <resolve		\ resolve branch back
  >resolve  ;  immediate		\ resolve branch out

: leave		\ -- ; immediate because standard says so
  compile (leave)  ;  immediate

: ?leave	\ -- ; immediate because standard says so
  compile (?leave)  ;  immediate


\ begin until end again repeat if endif then else while

: begin		\ -- addr 1
  ?comp  <mark 1  ;  immediate

: until		\ addr 1 --
  1 ?pairs  compile ?branch <resolve  ;  immediate

: again		\ addr 1 --
  1 ?pairs  compile branch <resolve  ;  immediate

: while		\ addr1 1 -- addr1 addr2 3
  1 ?pairs  compile ?branch >mark  3  ;  immediate

: repeat	\ addr1 addr2 3 --
  3 ?pairs
  swap compile branch <resolve
  >resolve  ;  immediate
\ if endif then (else) case of ?of endof endcase case-end next-case

: if		\ addr 2 --
  compile ?branch >mark  2 ;  immediate

: endif		\ addr 2 --
  ?comp  2 ?pairs  >resolve  ;  immediate
: then		\ addr 2 --
  [compile] endif ; immediate

internal
: (else)	\ addr1 -- addr2 ; used by ELSE and ENDOF
  compile branch >mark
  swap >resolve  ;
external

: else		\ addr1 2 -- addr2 2
  2 ?pairs  (else)  2  ;  immediate

internal
: fix-exits	\ nn...n0 -- ; resolves ENDOFs from CASE statement
  begin  sp@ csp @ <>			\ while stack not at level of CASE
    while  >resolve			\ resolve forward branch from ENDOF
  repeat  ;
external

: case		\ -- addr 4
  ?comp  csp @  <mark  !csp  4  ;  immediate

: of		\ 4 -- addr 5
  4 ?pairs  compile (of) >mark  5  ;  immediate

: endof		\ addr1 5 -- addr2 4
  5 ?pairs  (else)  4  ;  immediate

: endcase	\ addrn .... addr0 4
  4 ?pairs  compile drop
  fix-exits  drop  csp !  ;  immediate

((
: ?of		\ 4 -- addr 5
  4 ?pairs  compile ?branch >mark  5  ; immediate

: end-case	\ addrn .... addr0 4
  4 ?pairs  fix-exits  drop  csp !  ;  immediate

: next-case	\ addrn .... addr0 4
  4 ?pairs  compile branch  0 ,
  fix-exits  here 2- !  csp !  ;  immediate
))
\ ' ['] [compile]

: '		\ -- cfa ; ' <name> returns CFA of <name>
  bl word find 0= 0 ?error  ;

: [compile]	\ -- ; [compile] <name> compiles CFA of <name>
  ' ,  ;  immediate

: [']		\ -- ; ['] <name> compiles CFA as literal
  ' [compile] literal  ;  immediate


\ ( \ ", "" abort" .( ."

hex

: (		\ -- ; comment terminated by ')'
  029 word drop  ;  immediate

: \		\ -- ; comment to end of line
  blk @					\ in block?
  if  >in @ c/l / 1+ c/l * >in !	\ y: skip to 64 byte boundary
  else  0 word drop			\ n: skip to end of line
  endif  ;  immediate

: ",		\ -- ; compile string cccc" into dictionary
  022 word c@ 1+ allot  align  ;

: "" 		\ -- [compiling] ; -- addr [exec] ; "" <text>" compiles text
  state @				\ compiling?
  if  compile (")  ",			\ y: compile string into dictionary
  else  022 word pad over c@ 1+ move	\ n: copy string to PAD
        pad				\ return address of PAD
  endif  ;  immediate

: abort"	\ -- ; compile action and string
  compile (abort")  ",  ;  immediate

: .(		\ -- ; printed commed
  029 word $.  ;  immediate

: ."		\ -- ; printed string
  state @				\ compiling?
  if  compile (.")  ",			\ y: compile action and string
  else  022 word $.			\ n: just display string
  endif  ;  immediate
\ .name hash-thread                                jon 17/11/88

hex

: .name		\ nfa -- ; display word's name
  count 01F and  type space  ;

: hash-thread	\ voc-cfa $addr -- thread-addr ; given vocabulary and string,
		\ determine which thread to use
  dup c@ swap 1+ c@ +			\ take string length & first char
  swap					\ get vocabulary CFA
  dup >#threads @ 1-  rot and  2*	\ mask made from #THREADS-1
  swap >threads +  ;			\ form thread addr


\ header $create create

hex

internal
: header	\ -- ; lay down header, using string at HERE+2
  current @ here 2+ hash-thread         \ find thread-addr
  dup @ ,                               \ lay old thread (link field)
  here swap !                           \ link this word
  here last !                           \ this is latest word
  here dup c@ width @ min 1+ allot      \ reserve space
  080 swap set-bit			\ set high bit
  here 1 and 0=                         \ force CFA to be ODD
  if  BL c,  endif  ;                   \  so PFA is EVEN

: $create	\ addr -- ; build dictionary header using given string
		\ N.B. dictionary must already be word aligned
  here 2+ over c@ 1+ move               \ copy string to here+2
  header docreate,  ;			\ lay header and JP DOCREATE
external

: create	\ -- ; CREATE <name> builds dictionary entry
  align					\ link field must be on even boundary
  bl word                               \ fetch data from i/p
  dup count upper                       \ convert to upper case
  dup current @ v-find                  \ see if name exists
  if  >name cr .name ." is redefined "  \ yes, say so
  else  drop  endif                     \ no
  $create  ;                            \ make header and cfa

\ (init) .free cold

hex

internal
: (init)        \ -- ; main initialisation
  init-ram dup 2+  ram-start  rot @ cmove  \ initialise ram
  init-u0 up !                          \ set up user pointer
  user-reset s0 010 cmove  blk off      \ initialise user vars
  init-tib 'tib !                       \
  init-fence fence 6 cmove              \ fence, dp, voc-link
  only forth definitions decimal  ;	\ set up search order
external

: .free		\ -- ; display free space in system
  here appl-rom u>			\ check whether in application
  here appl-rom-end u<  and
  if  appl-rom-end  else  em  endif	\ set appropriate top of RAM
  here - u. ." bytes free"  ;

L: STRT         ] COLD [        	\ cold start word

: startup	\ -- ;
  (init)				\ initialisation
  STRT @ execute			\ run startup word
;

: cold          \ -- ; cold start
  (init)                                \ set up, run user init
  init-ser                              \ initialise serial i/o
\ init-pages                            \ initialise paging
\ init-multi                            \ initialise tasking
\ relink                                \ link application ROM
  CR .cpu ."  vx.yz"			\ sign on
[ usrver here 2 - c!			\ insert version numbers
  mpe-ver here 3 - c!
  mpe-rel here 5 - c! ]
  cr  .free  cr				\ show free space
  cr  ."   ok"  abort  ;		\ go

\ halt? .......................................... jon 19/08/88

: halt?		\ -- t/f ; true if key pressed and not space
  key? dup                              \ a key ? no, continue
  if                                    \ yes,
    drop key bl =                       \  was it the space bar
    if                                  \  yes,
      key bl <>                         \   wait for key, space
     else -1                            \   continues
    endif                               \  no,
  endif  ;				\   halt.


\ words using a thread table                       jon 19/08/88

internal
: max-nfa       \ -- addr nfa ; returns addr and top nfa
  0 0                                   \ clear result
  pad context @ >#threads @ 2* bounds   \ for all threads
  do  i @  over u>                      \ if greater
      if  2drop  i  i @   endif         \ replace current set
  2 +loop  ;                            \ step on
external

also root definitions

: words		\ -- ; display list of words in CONTEXT dictionary
  context @ 0=				\ protect against NULL vocabulary
  if  exit  endif
  ." in "  context @ dup >name .name	\ display vocabulary name
  cr
  >threads pad		                \
  context @ >#threads @ 2* cmove        \ initialise threads
  begin                                 \
    max-nfa dup                         \ while still one left
    halt? 0= and                        \ and no stop command
   while                                \
    out @ c/l > if  cr  endif           \ new line ?
    dup .name                           \ show name
    n>link @ swap !                     \ find next
  repeat 
  2drop  ;

previous definitions

\ all done

decimal

;p
