\ MPE 80C196 PowerBoard target control file          sfp 04/12/90

\ Released for use with the MPE Forth Cross Compiler by:

\ MicroProcessor Engineering
\ 133 Hill Lane
\ Shirley
\ Southampton SO1 5AF
\ England

\ tel: (+44) 703 631441   (international)
\      0703 631441        (domestic)


only forth definitions decimal

all from-file macros			\ load assembler macros
					\ and register definitions 01Ah..02Bh

\  $320 Emu-Base				\ emulator i/o addr
  e27512 16bit Output-Emulator		\ define EPROM & Width

Floats

\ turn compiler on
CROSS-COMPILE

only forth definitions

  prefix			\ assembler opcodes first
  align-odd			\ headers word aligned
\  no-log			\ no output log

\ memory definitions                               sfp 10/05/91
hex
    $0000 $3FFF    KERNEL     KERN		\ Kernel ROM
    $8000 $BFFF 0  KERNEL-RAM KERN-DATA		\ Kernel RAM
          $C000    MEM-END			\ End of RAM

    $0000 IN-EMULATOR KERN			\ Output kernel to emulator

    $A000 Equ appl-rom				\ Define application area
    $C000 Equ appl-rom-end			\ both start and finish

use-code KERN					\ Select code page
use-data KERN-DATA				\ Select data page
\ Page definition

$4000 Equ page-start			\ start of EPROM that switches
$7FFF Equ page-end  			\ end of EPROM that switches

Page-end Page-start - 1+ Equ Page-size	\ size of each page



$0008 Equ #tasks			\ number of tasks, at least 1
					\ each task needs 0100 bytes
					\ this space is reserved first
$0080 Equ User-size			\ size of each tasks user area
$0040 Equ SP-size			\ size of each tasks data stack
$0040 Equ RP-size			\ size of each tasks return stack

$0100 Equ Tib-len			\ terminal i/p buffer length

\ Calculate memory map

User-size SP-size + Equ Task-s0		\ initial offset of data stack
Task-s0 RP-size + Equ Task-r0		\ initial offset of return stack

Task-r0 Equ per-task			\ sytem area size for each task

#tasks per-task * Equ taskram		\ space used for task pages
$0100 Equ intram			\ space used by interrupt page

em	\ length            name
         per-task - dup equ INT-INIT-U0	\ base of interrupt area
#tasks per-task * - dup equ INIT-U0	\ base of task user area
          tib-len - dup equ INIT-TIB	\ base of TIB
drop

\ Task 0, the initial task, has its stacks in the on-chip RAM for speed.
0F0 Equ init-r0         \ top of data stack
0C0 Equ init-s0         \ top of return stack
0FE Equ task-init-r0    \ top of data stack in task area
0C0 Equ task-init-s0    \ top of return stack in task area

\ Alternate Equates to put stacks in external RAM for main task
\ Init-u0 Task-r0 + Equ Init-r0		\ top of return stack
\ Init-u0 Task-s0 + Equ Init-s0		\ top of data stack


\ The user area, PAD, and two stacks are in one page per task
\ 000:03F       User variables
\ 040           PAD     the offset is defined in KERNEL.FTH
\ 080:0BF       Data stack
\ 0C0:0FF       Return stack
	       
\
\ Page 0 Register file allocation.
\
\ Registers 0..019h are the SFRs which are defined in SFR196.FTH.
\ Registers 01A..02Bh are used for the Forth virtual machine and
\ the PLM register set which is used as scratch space by Forth.
\
\ Other registers are defined here.
$2C Equ curtask				\ pointer to TCB of current task
$2E Equ task#				\ current task number
$2F Equ spshadow			\ shadow for SP_STAT register
$30 Equ rxchar				\ character received

\
\ Calculate baud rate using system clock rate
\
\ Baudreg = xtal/(16*baud) - 1
\
decimal
10,000000				\ clock speed
  16 mu/mod rot drop			\ divide by 16
9600
  um/mod nip				\ divide by baud rate
  1-					\ subtract one
  $8000 or				\ XTAL1 clock
  Equ sysbaud				\ set baud rate
  
\ pull in the files                                 \  01/10/91

$2000 $80 $FF fill				\ wipe vector area
$2080 org					\ skip vector area
  decimal  all from-file sfr196			\ Special Function Registers
  decimal  all	from-file startup		\ start up code
  decimal  all  from-file cd196kb		\ main code defs.
  decimal  all  from-file kernel		\ Forth high level kernel
$200 org
  decimal  all  from-file drivers\sci196i	\ On chip UART
  decimal  all  from-file int196		\ interrupt handler
  decimal  all  from-file multi96		\ multi-tasker
  decimal  all  from-file romtools		\ dump
  decimal  all  from-file page.fth		\ page switcher

\ Define pages

1 Constant Page1    			\ page-id for page 1
2 Constant Page2    			\ page-id for page 2
3 Constant Page3    			\ page-id for page 3

\ Define three pages, all at the same address
   Page-start Page-end Page1 CODE-PAGE FIRST-PAGE	\ Define page 1
   Page-start Page-end Page2 CODE-PAGE SECOND-PAGE	\ Define page 2
   Page-start Page-end Page3 CODE-PAGE THIRD-PAGE	\ Define page 3

\ Output all pages to emulator
   Page-size Page1 * IN-EMULATOR FIRST-PAGE	\ Output page 1 to emulator
   Page-size Page2 * IN-EMULATOR SECOND-PAGE	\ Output page 2 to emulator
   Page-size Page3 * IN-EMULATOR THIRD-PAGE	\ Output page 3 to emulator

\ example application

  decimal all from-file pagecode.fth

\ clean up                                         sfp 21/12/90

: FORTH-83  ;           \ final word

hex
     0FB  02018 !       \ chip configuration byte
                        \ unlimited wait states
                        \ ALE, WRH- & WRL-, 16 bit, no powerdown
     027  0201A c!      \ JMP SELF opcode
     0FE  0201B c!

decimal

FINIS

