\ ROMFORTH application linkage tools       

((
Copyright (c) 1991

MicroProcessor Engineering
133 Hill Lane
Southampton SO1 5AF
England

tel: (+44) 703 631441
fax: (+44) 703 339691

First Release: 29/04/91

Changes:	See RELEASE.DOC for full details
))

\ setup ................................           sfp 12/09/89

hex 
\ Define address of ROM/RAM area in which applications
\ are to be developed. This area is RAM during development.
\ When development is complete, the word SETUP is executed
\ to initialise the memory area.
\ The compiled image is then downloaded to the PC using the
\ tools in the ROMFORTH directory. The image is then programmed
\ into EPROM, which can then replace the RAM.
\ If the word RELINK is added to COLD, the application EPROM
\ will be automatically linked in when the target powers up.
\
appl-rom constant ROM                   \ start of ROM/RAM area


\ r-p
\ If your processor has separate CODE and DATA address spaces,
\ it has a HARVARD architecture.
\ DO NOT INCLUDE THIS PAGE for HARVARD CPUs e.g. 8031

\ The working RAM area has the interactive RAM dictionary at
\ the bottom and the application buffer space at the top.
\ Define the initial value of R-P to be the highest available
\ free location in the RAM area. Application variable and
\ buffer space will then be allocated downwards from this
\ address. When creating an application to be ROMmed,
\ download the code for BUFFER and the redefined VARIABLE
\ first of all.

variable r-p    EM r-p !                \ top of RAM area

\ setup ................................           sfp 13/09/89

hex

: setup         \ cfa -- ; sets for ROM generation
  forth definitions  055aa rom !        \ set marker
  rom 2+ !				\ set CFA to execute
  latest rom 4 + !			\ set last NFA
  r-p @  rom 6 + !			\ set top of RAM
  dp @   rom 8 + !			\ set end of dictionary
  ['] forth >#threads                   \ start of FORTH threads
  rom 0A + 				\ storage area
  over @ 2* 2+ cmove                    \ copy threads
  ;


\ use-setup-data                                   sfp 13/09/89

internal

: use-setup-data        \ -- ; use data in setup area
  rom 0A + 				\ storage area
  ['] forth >#threads                   \ start of FORTH threads
  over @ 2* 2+ cmove                    \ copy threads
  rom 6 + @ r-p !			\ top of RAM
  rom 4 + @ last !			\ last word defined
  rom 2 + @ ?dup			\ if CFA set
  if  execute  endif  ;                 \  execute it


\ link-rom                                         sfp 13/09/89

: link-rom      \ -- ; link in application ROM area
  use-setup-data                        \ copy data from header
  Cr ." Application ROM installed"  ;   \


\ link-ram                                         sfp 21/03/89

hex

: link-ram      \ -- ; link in application RAM area
  0AA55 rom !  rom @ 0AA55 =            \ see if RAM present
  if  rom 040 + dup			\ yes, reserve first 64
      dp !  fence !                     \ and reset pointers
      Cr ." Application RAM installed"  \ tell user
  endif  ;

\ link-nv-ram                                      sfp 13/09/89

: link-nv-ram   \ -- ; link in for NV-RAMs
  rom 8 + @
  dup dp ! fence !                      \ end of dictionary
  use-setup-data                        \ copy data from header
  Cr ." Application NV-RAM installed" ; \

external
\ relink ...............................           sfp 13/09/89

hex

: relink        \ -- ; re-link application ROM/RAM if there
  rom @ 055aa =                         \ see if ROM present
  if  0AA55 rom !                       \ ROM or NV-RAM ?
      rom @ 0AA55 =                     \ true if NV-RAM
      055AA rom !                       \ restore header
      if  link-nv-ram                   \ link in NV-RAM
      else  link-rom                    \ link in ROM area
      endif                             \
  else  link-ram                        \ no, attempt RAM area
  endif  ;

((
Note that distribution of open Forth systems containing this
code is only permitted after a ROMFORTH license agreement has
been signed by the distributor and MPE.
))
\ all done

decimal

;p
